#include <boost/thread/mutex.hpp>
#include "cache.hpp" 


Cache::Cache()
{   
    cacheSize_ = 512;    
    this->Cache_ = new lru11::Cache<string, uint32_t>(cacheSize_, 0);
    containerPool_ = (uint8_t**) malloc(cacheSize_ * sizeof(uint8_t*));
    for (size_t i = 0; i < cacheSize_; i++) {
        containerPool_[i] = (uint8_t*) malloc(config.getMaxContainerSize() * sizeof(uint8_t));
    }
    currentIndex_ = 0;
    fprintf(stderr, "Container Cache Size: %lu\n", this->cacheSize_);
}

void Cache::insertToCache(string& name, uint8_t* data, uint32_t length)
{
    {
        boost::unique_lock<boost::shared_mutex> t(this->mtx);
        if (Cache_->size() + 1 > cacheSize_) {
            // evict a item
            uint32_t replaceIndex = Cache_->pruneValue();
            // fprintf(stdout, "Cache evict: %u\n", replaceIndex);
            memset(containerPool_[replaceIndex], 0, config.getMaxContainerSize());
            memcpy(containerPool_[replaceIndex], data, length);
            Cache_->insert(name, replaceIndex);
        } else {
            // directly using current index
            // fprintf(stdout, "Cache use: %lu\n", currentIndex_);
            memset(containerPool_[currentIndex_], 0, config.getMaxContainerSize());
            memcpy(containerPool_[currentIndex_], data, length);
            Cache_->insert(name, currentIndex_);
            currentIndex_++;
        }
    }
}

bool Cache::existsInCache(string& name)
{
    bool flag = false;
    {
        boost::shared_lock<boost::shared_mutex> t(this->mtx);
        flag = this->Cache_->contains(name);
    }
    return flag;
}

uint8_t* Cache::getFromCache(string& name)
{   
    {
        boost::shared_lock<boost::shared_mutex> t(this->mtx);
        uint32_t index = this->Cache_->get(name);
        return containerPool_[index]; 
    }
    
}

Cache::~Cache() {
    for (size_t i = 0; i < cacheSize_; i++) {
        free(containerPool_[i]);
    } 
    free(containerPool_);
    delete Cache_;
}